#!/usr/bin/env python3

"""Handle configuration options and command line options."""

import functools
import os
import pathlib
import sys
import tempfile

import configargparse
import importlib_resources


__all__ = ["Config"]


PACKAGE = __package__.split(".")[0]
CONFIG_FILE_TEMPLATE = importlib_resources.files(f"{PACKAGE}").joinpath(
    f"{PACKAGE}.yml.template"
)

if "HOME" not in os.environ:  # e.g., testing environment or container
    os.environ["HOME"] = "."


class Config:
    """Load configuration from config files or command line arguments."""

    _instance = None  # stores singleton instance

    def __init__(self):
        """Load configuration from config files or command line arguments."""

        self.argparser.add(
            "-v",
            "--verbose",
            help="Enable verbose output from R5 and r5py",
            action="store_true",
        )

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super(Config, cls).__new__(cls)
        return cls._instance

    @property
    def arguments(self):
        """
        Arguments passed from command line or config file.

        Ignores `--help`: can be used while not all modules have added arguments.
        """
        return self.get_arguments(ignore_help_args=True)

    @property
    def argparser(self):
        try:
            argparser = configargparse.get_argument_parser(
                prog=PACKAGE,
                description=sys.modules[PACKAGE].__doc__,
                default_config_files=self.CONFIG_FILES,
            )
        except ValueError:  # has been instantiated, already
            argparser = configargparse.get_argument_parser()
        return argparser

    @functools.cached_property
    def CACHE_DIR(self):
        cache_dir = (
            pathlib.Path(
                os.environ.get("LOCALAPPDATA")
                or os.environ.get("XDG_CACHE_HOME")
                or (pathlib.Path(os.environ["HOME"]) / ".cache")
            )
            / PACKAGE
        )
        cache_dir.mkdir(parents=True, exist_ok=True)
        return cache_dir

    @functools.cached_property
    def CONFIG_FILES(self):
        config_files = [
            pathlib.Path(f"/etc/{PACKAGE}.yml"),
            pathlib.Path(
                os.environ.get("APPDATA")
                or os.environ.get("XDG_CONFIG_HOME")
                or (pathlib.Path(os.environ["HOME"]) / ".config")
            )
            / f"{PACKAGE}.yml",
        ]

        # write a template configuration file to possible locations
        for config_file in config_files:
            self._copy_config_file_template(config_file)

        # argparse does not understand pathlib.Path
        config_files = [str(config_file) for config_file in config_files]

        return config_files

    @staticmethod
    def _copy_config_file_template(destination_path):
        """
        Try to copy a configuration file template.

        Arguments:
        ----------
        destination_path : pathlib.Path
            Where could/should a configuration file exist?
        """
        if not destination_path.exists():
            try:
                destination_path.parent.mkdir(parents=True, exist_ok=True)

                with importlib_resources.as_file(CONFIG_FILE_TEMPLATE) as template:
                    destination_path.write_text(template.read_text())

            except (
                FileNotFoundError,
                FileExistsError,
                IsADirectoryError,
                PermissionError,
            ):
                pass

    def get_arguments(self, ignore_help_args=False):
        """Parse arguments passed from command line or config file."""
        return self.argparser.parse_known_args(ignore_help_args=ignore_help_args)[0]

    @functools.cached_property
    def TEMP_DIR(self):
        parent_dir = self.arguments.temporary_directory
        temp_dir = pathlib.Path(tempfile.mkdtemp(prefix=self.PACKAGE), dir=parent_dir)
        return temp_dir

    PACKAGE = PACKAGE



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2019 Christoph Fink, University of Helsinki
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 3
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>.

""" Reminds me if it rains """

import statistics
import sys

import geocoder

from .config import Config
from .emailsender import EMailSender
from .precipitationchecker import PrecipitationChecker


__all__ = ["remind_me_if_it_rains"]

# How many hours to look into the future?
N_HOURS = 14


config = Config()

config.argparser.add(
    "--place",
    help="location of precipitation forecast",
    required=True,
)

config.argparser.add(
    "--average_precipitation_rate_threshold",
    help="send notifications when the average precipitation rate exceeds this value",
    type=float,
    default=0.1,
)

config.argparser.add(
    "--max_precipitation_rate_threshold",
    help="send notifications when the maximum precipitation rate exceeds this value",
    type=float,
    default=0.5,
)

config.argparser.add(
    "--email-subject",
    help="Subject line for the e-mail notification",
    default="Pack your rain gear",
)

config.argparser.add(
    "--email-message",
    help="message body of the e-mail notification",
    default=(
        "The average forecast precipitation rate for today is {a:0.2f}, "
        "maximum {m:0.2f} mm/h."
    ),
)

config.argparser.add(
    "--email-from",
    help="e-mail address to send notification messages from",
)

config.argparser.add(
    "--email-to",
    help="receiver e-mail address for notification messages",
)

config.argparser.add(
    "--smtp-host",
    help="SMTP server to send notifications via (host:port)",
    default="localhost:587",
)

config.argparser.add(
    "--smtp-user",
    help="User as which to log in to the SMTP server",
    default="",
)

config.argparser.add(
    "--smtp-password",
    help="Password used to log in to the SMTP server",
    default="",
)

config.argparser.add(
    "--user-agent",
    help="User-Agent header to report to API services",
    default="python-wolkenbruch (https://github.com/christophfink/wolkenbruch/)",
)


def remind_me_if_it_rains():
    """Remind me if rain is forecast"""

    try:
        lat, lon = geocoder.osm(
            config.arguments.place,
            headers={"User-Agent": config.arguments.user_agent},
        ).latlng
    except (TypeError, ValueError) as exception:
        raise RuntimeError(
            f"Could not find location ‘{config.arguments.place}’"
        ) from exception

    hourly_precipitation_rates = PrecipitationChecker(
        lat,
        lon,
        user_agent=config.arguments.user_agent,
    ).hourly_precipitation_rates[:N_HOURS]

    average_precipitation_rate = statistics.fmean(hourly_precipitation_rates)
    max_precipitation_rate = max(hourly_precipitation_rates)

    if config.arguments.verbose:
        print(
            (
                "Average precipitation rate in {place:s} is {a:0.2f} mm/h "
                + "over the next 14 hours, maximum {m:0.2f}. "
            ).format(
                place=config.arguments.place,
                a=average_precipitation_rate,
                m=max_precipitation_rate,
            ),
            file=sys.stderr,
            end="",
        )

    if (
        average_precipitation_rate
        > config.arguments.average_precipitation_rate_threshold
        or max_precipitation_rate > config.arguments.max_precipitation_rate_threshold
    ):
        EMailSender(
            config.arguments.email_from,
            config.arguments.email_to,
            config.arguments.email_subject,
            config.arguments.email_message.format(
                a=average_precipitation_rate,
                m=max_precipitation_rate,
            ),
            config.arguments.smtp_host,
            config.arguments.smtp_user,
            config.arguments.smtp_password,
        ).send_message()

        if config.arguments.verbose:
            print(
                "Sending reminder to {:s} ".format(config.arguments.email_to),
                file=sys.stderr,
            )
    else:
        if config.arguments.verbose:
            print(
                "NOT sending reminder to {:s} ".format(config.arguments.email_to),
                file=sys.stderr,
            )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2019 Christoph Fink, University of Helsinki
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 3
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>.

""" Sends e-mails (convenience wrapper around `smtplib` and `email.message` """


import email.message
import smtplib


__all__ = ["EMailSender"]


class EMailSender:
    """Sends e-mails

    Args:
        from_addr:  Sender address
        to_addr:    Receiver address
        subject:    Subject line
        body:       e-mail text
        smtp_host:  host name/ip address to use as an SMTP server,
                    can contain port in the form [host]:[port]
        smtp_user:  SMTP user name if authentication required
        smtp_password: SMTP password if authentication required
        starttls:   use TLS (default: True)
    """

    def __init__(
        self,
        from_addr="",
        to_addr="",
        subject="",
        body="",
        smtp_host="",
        smtp_user="",
        smtp_password="",
        starttls=True,
        *args,
        **kwargs,
    ):
        """
        Sends e-mails

        Args:
            from_addr:  Sender address
            to_addr:    Receiver address
            subject:    Subject line
            body:       e-mail text
            smtp_host:  host name/ip address to use as an SMTP server,
                        can contain port in the form [host]:[port]
            smtp_user:  SMTP user name if authentication required
            smtp_password: SMTP password if authentication required
            starttls:   use TLS (default: True)
        """
        self.from_addr = from_addr
        self.to_addr = to_addr
        self.subject = subject
        self.body = body

        self.smtp_host = smtp_host
        self.smtp_user = smtp_user
        self.smtp_password = smtp_password
        self.starttls = starttls

    def send_message(self):
        """Sends a message to `to_addr`"""
        with smtplib.SMTP(self.smtp_host) as smtp:
            if self.starttls:
                smtp.starttls()
            if self.smtp_user != "" and self.smtp_password != "":
                smtp.login(self.smtp_user, self.smtp_password)

            message = email.message.EmailMessage()
            message["From"] = self.from_addr
            message["To"] = self.to_addr
            message["Subject"] = self.subject
            message.set_content(self.body)

            smtp.send_message(message)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2019 Christoph Fink, University of Helsinki
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 3
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>.

""" Checks the precipitation over the next n hours at a given location """


import requests


__all__ = ["PrecipitationChecker"]


class PrecipitationChecker:
    """Check the forecast precipitation at a location

    Args:
        lat:        Latitude
        lon:        Longitude
        n_hours:    calculate the average precipitation
                    rate for the next `n_hours` hours
    """

    API_VERSION = "2.0"
    API_ENDPOINT = (
        f"https://api.met.no/weatherapi/locationforecast/{API_VERSION:s}/compact"
    )

    def __init__(self, lat, lon, user_agent, n_hours=14, *args, **kwargs):
        """
        Check the forecast precipitation at a location.

        Args:
            lat:        Latitude
            lon:        Longitude
            user_agent: pass this User-Agent header to the API endpoint
            n_hours:    calculate the average precipitation
                        rate for the next `n_hours` hours
        """
        self.lat = lat
        self.lon = lon
        self.user_agent = user_agent
        self.n_hours = n_hours

        self.precipitation = []

    def _fetch_weather_forecast(self):
        forecast = requests.get(
            self.API_ENDPOINT,
            params={
                "lat": self.lat,
                "lon": self.lon,
            },
            headers={"User-Agent": self.user_agent},
        )
        forecast = forecast.json()

        self.precipitation = [
            timestamp["data"]["next_1_hours"]["details"]["precipitation_amount"]
            for timestamp in forecast["properties"]["timeseries"]
            if "next_1_hours" in timestamp["data"]
        ]

    @property
    def hourly_precipitation_rates(self):
        """
        Retrieve precipitation rates by hour.

        How much precipitation is forecast
        for the next n hours in mm/h
        """
        if not self.precipitation:
            self._fetch_weather_forecast()
        return self.precipitation
